
local CorralFrame  = ZooKeeper:NewClass('Frame')
ZooKeeper.Frame = CorralFrame

--local references
local _G = getfenv(0)
local L = LibStub('AceLocale-3.0'):GetLocale('ZooKeeper')

--frame constructor
local lastID = 1
function CorralFrame:New(titleText, db, key)
    local f = self:Bind(CreateFrame('Frame', format('ZooKeeperFrame%d', lastID), UIParent, 'ZooKeeperCorralTemplate'))
    f.db = db

    --constants
    f.MIN_WIDTH = 280
    f.MIN_HEIGHT = 50
    f.BUTTON_SIZE = 37
    f.FRAME_PADDING = 12
    f.TOP_PADDING = 40

    f:SetScript('OnShow', self.OnShow)
    f:SetScript('OnHide', self.OnHide)
    f:UpdateBackdrop()
    f:SetScale(f.db.scale)

    f.titleText = titleText

    if f.db.lastViewed == TYPE_MOUNTS then
       f.isMount = true
    else
       f.isMount = false
    end

    f.mountButtons = {}
    f.companionButtons = {}
    f.mountFilter = {}
    f.companionFilter = {}

    self:SetWidth(f.MIN_WIDTH)
    self:SetHeight(f.MIN_HEIGHT)

    f.searchFilterBox = _G[f:GetName() .. 'Search']

    --update if pets are shown or not
    f:InitFilter()
    f:UpdatePets()

    --place the frame
    f:LoadPosition()

    lastID = lastID + 1

    table.insert(UISpecialFrames, f:GetName())

    return f
end

function CorralFrame:UpdateBackdrop()
    self:SetBackdropColor(0.2, 0.2, 0.2, self.db.opacity)
end


--[[
    Drag Icon
--]]

function CorralFrame:DragIconOnEnter(dragIcon)
    GameTooltip:SetOwner(dragIcon, 'ANCHOR_LEFT')
    GameTooltip:AddLine(L.DragIconHelpText)
    GameTooltip:Show()
end

--[[
    Companion/Mount Toggle
--]]

function CorralFrame:OnTypeToggleClick(toggle, button)
    if button == 'LeftButton' then
        self:ToggleCorralType()
    end
end

function CorralFrame:OnTypeToggleEnter(toggle)
    GameTooltip:SetOwner(toggle, 'ANCHOR_LEFT')

    if self.isMount == true then
        GameTooltip:AddLine(L.CompanionToggle)
    else
        GameTooltip:AddLine(L.MountToggle)
    end
    GameTooltip:Show()
end

function CorralFrame:SavePosition(point, parent, relPoint, x, y)
    if point then
        if self.db.position then
            self.db.position[1] = point
            self.db.position[2] = parent
            self.db.position[3] = relPoint
            self.db.position[4] = x
            self.db.position[5] = y
        else
            self.db.position = {point, parent, relPoint, x, y}
        end

        self:SetUserPlaced(true)
    else
        self.db.position = nil
        self:SetUserPlaced(false)
    end
end

function CorralFrame:LoadPosition()
    if self.db.position then
        ZooKeeper:Print("loaded position");
        self:SetPoint(unpack(self.db.position))
        self:SetUserPlaced(true)
    else
        self:SetPoint("CENTER", "UIParent", "CENTER", 100, 200);
        self:SetUserPlaced(false)
    end
end


--[[
    Filtering
--]]

-- Just initialize it to what it was last time
function CorralFrame:InitFilter()
    local filter = self:GetFilter()

    if filter then
        self.searchFilterBox:SetText(filter)
        self:SetFilterSet(filter)
    else
        self.searchFilterBox:SetText('')
    end
end

function CorralFrame:SetFilter(key, value)
    local type = self:GetCurrentMode()

    local filter = self:GetFilter()

    if not filter or filter ~= value then
        self.db.searchFilter[type] = value
        self:SetFilterSet(value)
        return true
    end
end

function CorralFrame:GetFilter()
    local type = self:GetCurrentMode()

    if not self.db.searchFilter then
        self.db.searchFilter = {}
    end

    return self.db.searchFilter[type]
end

function CorralFrame:GetFilterSet()
    local returnSet

    if self.isMount == true then
        returnSet = self.mountFilter
    else
        returnSet = self.companionFilter
    end

    return returnSet
end

function CorralFrame:SetFilterSet(value)
    local petID, creatureID, creatureName, spellID, icon, active;
    local orderSet = self:GetOrderSet()
    local filterSet = {}
    local counter = 1

    if value then
        for i = 1, #orderSet do
            petID = orderSet[i]

            creatureID, creatureName, spellID, icon, active = GetCompanionInfo(self.db.lastViewed, petID)
            creatureName = string.lower(creatureName)
            if string.match(creatureName, value) then
                filterSet[counter] = petID
                counter = counter + 1
            end
        end
    end

    if self.isMount == true then
        self.mountFilter = filterSet
    else
        self.companionFilter = filterSet
    end

    self:UpdatePets()
end

--[[
    Display
--]]

function CorralFrame:OnSizeChanged()
end

function CorralFrame:OnShow()
    PlaySound('igMainMenuOpen')
    self:UpdatePets()
end

function CorralFrame:OnHide()
    PlaySound('igMainMenuClose')
end

function CorralFrame:ToggleCorralType()
    if self:IsShown() then
        self:HideCurrentPets()
    end

    if self.isMount == true then
        self.isMount = false
        self.db.lastViewed = TYPE_COMPANIONS
    else
        self.isMount = true
        self.db.lastViewed = TYPE_MOUNTS
    end

    if self:IsShown() then
        self:InitFilter()
        self:UpdatePets()
    end
end

function CorralFrame:ToggleFrame()
    if not self:IsShown() then
        ZooKeeper:Print("Showing...");
        self:ShowFrame()
    else
        ZooKeeper:Print("Hiding...");
        self:HideFrame()
    end
end

function CorralFrame:ShowFrame(auto)
    if not self:IsShown() then
        ZooKeeper:Print("Showing........");
        ShowUIPanel(self)
        self.autoShown = auto or nil
    end
end

function CorralFrame:HideFrame(auto)
    if self:IsShown() then
        if not auto or self.autoShown then
            HideUIPanel(self)
            self.autoShown = nil
        end
    end
end

function CorralFrame:GetButtonSet()
    if self.isMount == true then
        return self.mountButtons
    else
        return self.companionButtons
    end
end

function CorralFrame:HideCurrentPets()
    local button
    local numPets = self:GetNumberPets()
    local buttonSet = self:GetButtonSet()

    for i = 1, numPets do
        button = buttonSet[i]
        if button then
            button:Hide()
        end
    end
end

function CorralFrame:GetOrderSet()
    local returnSet
    local numPets

    if isMount == true then
        returnSet = self.db.mountOrder
    else
        returnSet = self.db.companionOrder
    end

    -- ensure that every existing pet is in here somewhere
    numPets = self:GetNumberPets()
    for i = (#returnSet + 1), numPets do
        if not returnSet[i] then
            returnSet[i] = i
        end
    end

    return returnSet
end

function CorralFrame:GetOrCreateButton(buttonSet, i)
    local button

    button = buttonSet[i]
    if not button then
        button = self:CreateButton(i)
        buttonSet[i] = button
    end

    return button
end

function CorralFrame:UpdatePets()
    local numPets = self:GetNumberPets()
    local currentRow = 1
    local lastRow
    local currentColumn = 0
    local button
    local lastButton
    local newRow
    local buttonSet = self:GetButtonSet()
    local orderSet = self:GetOrderSet()
    local filterSet = self:GetFilterSet()

    for i = 1, numPets do
        if i > (currentRow * self.db.columns) then
            currentColumn = 1
            currentRow = currentRow + 1
            newRow = true
        else
            currentColumn = currentColumn + 1
            newRow = false
        end

        button = self:GetOrCreateButton(buttonSet, i)
        self:PositionButton(button, i, currentRow, newRow, lastRow, lastButton)
        self:UpdateButton(button, i, filterSet, orderSet)

        if newRow == true or i == 1 then
            lastRow = button
        end

        lastButton = button
    end

    self:CalculateAndSetSize()
end

function CorralFrame:GetCurrentMode()
    return self.db.lastViewed
end


function CorralFrame:GetNumberPets()
    local mode = self:GetCurrentMode()
    return GetNumCompanions(mode)
end

function CorralFrame:GetTotalRows()
    local numPets = self:GetNumberPets()
    local numRows = numPets / self.db.columns
    if numRows < 1 then
        numRows = 1
    end
    return numRows
end

function CorralFrame:GetVisibleRows()
    local numPets = self:GetNumberPets()
    local currentRow = 1
    local buttonSet = self:GetButtonSet()

    for i = 1, numPets do
        button = buttonSet[i]
        if button.petID and button.petID ~= nil then
            currentRow = button.row
        end
    end

    return currentRow
end

function CorralFrame:CalculateAndSetSize()
    local width = self.BUTTON_SIZE * self.db.columns
    width = width + (self.db.buttonPadding * (self.db.columns + 1))
    width = width + (self.FRAME_PADDING * 2)
    if width < self.MIN_WIDTH then
        width = self.MIN_WIDTH
    end

    local displayedRows = self:GetVisibleRows()
    if displayedRows < 1 then
        displaedRows = 1
    end

    local height = self.BUTTON_SIZE * displayedRows
    height = height + (self.db.buttonPadding * (displayedRows + 1))
    height = height + (self.FRAME_PADDING * 2) + self.TOP_PADDING
    if height < self.MIN_HEIGHT then
        height = self.MIN_HEIGHT
    end

    self:SetWidth(width)
    self:SetHeight(height)
end

